@@ -1,13 +1,15 @@
+from aiohttp.client_exceptions import ClientConnectorError
 import asyncio
 import datetime
 from io import BytesIO
+import logging
 import math
 import os
 from PIL import Image, ImageDraw, ImageFont
-import xml.etree.ElementTree as et
 
 from .ec_cache import CacheClientSession as ClientSession
 import dateutil.parser
+import defusedxml.ElementTree as et
 import imageio
 import voluptuous as vol
 
@@ -16,6 +18,8 @@ ATTRIBUTION = {
     "french": "Données fournies par Environnement Canada",
 }
 
+__all__ = ["ECRadar"]
+
 # Natural Resources Canada
 
 basemap_url = "http://maps.geogratis.gc.ca/wms/CBMT"
@@ -29,6 +33,12 @@ basemap_params = {
     "format": "image/png",
 }
 
+# Mapbox Proxy
+
+backup_map_url = (
+    "https://0wmiyoko9f.execute-api.ca-central-1.amazonaws.com/mapbox-proxy"
+)
+
 # Environment Canada
 
 precip_layers = {"rain": "RADAR_1KM_RRAI", "snow": "RADAR_1KM_RSNO"}
@@ -139,13 +149,14 @@ class ECRadar(object):
             "width": self.width,
             "height": self.height,
         }
-        self.base_bytes = None
+        self.map_image = None
         self.radar_opacity = kwargs["radar_opacity"]
 
         # Get overlay parameters
 
         self.show_legend = kwargs["legend"]
         if self.show_legend:
+            self.legend_layer = None
             self.legend_image = None
             self.legend_position = None
 
@@ -168,7 +179,11 @@ class ECRadar(object):
 
         if self._precip_setting in ["rain", "snow"]:
             self.layer_key = self._precip_setting
-        elif datetime.date.today().month in range(4, 11):
+        else:
+            self._auto_precip_type()
+
+    def _auto_precip_type(self):
+        if datetime.date.today().month in range(4, 11):
             self.layer_key = "rain"
         else:
             self.layer_key = "snow"
@@ -176,9 +191,27 @@ class ECRadar(object):
     async def _get_basemap(self):
         """Fetch the background map image."""
         basemap_params.update(self.map_params)
-        async with ClientSession(raise_for_status=True) as session:
-            response = await session.get(url=basemap_url, params=basemap_params)
-            self.base_bytes = await response.read()
+
+        try:
+            async with ClientSession(raise_for_status=True) as session:
+                response = await session.get(url=basemap_url, params=basemap_params)
+                base_bytes = await response.read()
+                self.map_image = Image.open(BytesIO(base_bytes)).convert("RGBA")
+
+        except ClientConnectorError:
+            logging.warning("NRCan base map could not be retreived")
+
+            try:
+                async with ClientSession(raise_for_status=True) as session:
+                    response = await session.get(
+                        url=backup_map_url, params=basemap_params
+                    )
+                    base_bytes = await response.read()
+                    self.map_image = Image.open(BytesIO(base_bytes)).convert("RGBA")
+            except ClientConnectorError:
+                logging.warning("Mapbox base map could not be retreived")
+
+        return
 
     async def _get_legend(self):
         """Fetch legend image."""
@@ -193,6 +226,7 @@ class ECRadar(object):
         self.legend_image = Image.open(BytesIO(legend_bytes)).convert("RGB")
         legend_width = self.legend_image.size[0]
         self.legend_position = (self.width - legend_width, 0)
+        self.legend_layer = self.layer_key
 
     async def _get_dimensions(self):
         """Get time range of available data."""
@@ -206,9 +240,7 @@ class ECRadar(object):
             )
             capabilities_xml = await response.text()
 
-        capabilities_tree = et.fromstring(
-            capabilities_xml, parser=et.XMLParser(encoding="utf-8")
-        )
+        capabilities_tree = et.fromstring(capabilities_xml)
         dimension_string = capabilities_tree.find(
             dimension_xpath.format(layer=precip_layers[self.layer_key]),
             namespaces=wms_namespace,
@@ -222,12 +254,6 @@ class ECRadar(object):
     async def _combine_layers(self, radar_bytes, frame_time):
         """Add radar overlay to base layer and add timestamp."""
 
-        # Overlay radar on basemap
-
-        if not self.base_bytes:
-            await self._get_basemap()
-
-        base = Image.open(BytesIO(self.base_bytes)).convert("RGBA")
         radar = Image.open(BytesIO(radar_bytes)).convert("RGBA")
 
         # Add transparency to radar
@@ -238,12 +264,19 @@ class ECRadar(object):
             radar_copy.putalpha(alpha)
             radar.paste(radar_copy, radar)
 
-        frame = Image.alpha_composite(base, radar)
+        # Overlay radar on basemap
+
+        if not self.map_image:
+            await self._get_basemap()
+        if self.map_image:
+            frame = Image.alpha_composite(self.map_image, radar)
+        else:
+            frame = radar
 
         # Add legend
 
         if self.show_legend:
-            if not self.legend_image:
+            if not self.legend_image or self.legend_layer != self.layer_key:
                 await self._get_legend()
             frame.paste(self.legend_image, self.legend_position)
 
@@ -289,6 +322,9 @@ class ECRadar(object):
         return await self._combine_layers(frame, latest)
 
     async def update(self):
+        if self.precip_type == "auto":
+            self._auto_precip_type()
+
         self.image = await self.get_loop()
 
     async def get_loop(self, fps=5):
